#!/usr/bin/perl

=head1 Name

 crawler - Simple crawler for personal site.

=cut

use strict;
use Getopt::Long;
# provide path to your custom dictionaries
#use lib qw (/u/megera/app/fts/test-suite);
use MyCrawler2;

$|=1;
my ($page, $errstr);
my %opts = ( maxpages => 10, debug => 0, op => \&check_op, help =>\&usage );
die("Try $0 --help\n") if !GetOptions( \%opts, 'op=s', 'dump=s', 'load=s', 'seed=s', 'maxpages=i', 'db=s', 'help', 'debug=i');

my $parent = $ARGV[0];

usage ( "No parent page specified !") if ( !$parent && $opts{op} ne 'reindex');

my @exclude_patterns = qw (
                    \.gif$ \.jpg$ \.gz$
                          );


my ( $dbname, $prefix ) = '';
( $dbname, $prefix ) = split(':',$opts{db});

my $crawler = MyCrawler->new(dbname   => $dbname,
                             maxpages => $opts{maxpages},
                             parent   => $parent,
                             debug    => $opts{debug},
                             op       => $opts{op},
                             patterns => \@exclude_patterns,
                             delay    => 0.001,
                             prefix   => $prefix,
                            );

print "Parent page:",$crawler->{parent},"\n";

my $uri;

if ( $crawler->{op} eq 'reindex' ) { # REINDEX, get links,tid from db
  $crawler->getlinks_db();
  $uri = $crawler->next_url(); 
} elsif ( $crawler->{op} eq 'add' and $opts{load}) { # load links from file

 if ( open(LINKS, "<$opts{load}") ) {
   my $loaded = $crawler->load_links(\*LINKS);
   close(LINKS);
   if ( $loaded ) {
     $crawler->{maxpages} ||= $loaded;   # if not specified !
   } else {
     warn "Problems with loading links from $opts{load} !\n";
   }
 } else {
   warn "Unable to open $opts{load} !\n";
 }  
 $uri = $crawler->next_url();

} else {
  $uri = $opts{seed} || $crawler->{parent};
  print "Seed page:$uri\n";
}

my $ua = $crawler->{ua};
my $totalsize = 0;
my $npage = 0;
while ( defined $uri && $npage < $crawler->{maxpages} ) {
 if ( $crawler->{op} ne 'reindex' || $crawler->{expire}->{$uri} < time()  ){
   $page = $crawler->fetch_page ( $ua, $uri );
   if ( $page->{success} && $page->{content_type} =~ /^text\// ) {
     if ( $crawler->{op} eq 'noindex' ) {
        $crawler->{tid}++;
        $npage++;
     } else {
       if (    !( $crawler->{op} eq 'add' && $crawler->url_exists($page->{uri}) )
            && !defined $crawler->md5_exists($page->{md5})
          ) {
          $errstr = $crawler->index($crawler->{tid},$page);
          if ( $errstr ) {
             warn "$errstr:$uri\n";
          } else {
             $npage++;
             $totalsize += $page->{content_length};
          }
       }
     }
     $crawler->getlinks($page->{base}, $page->{content}) if ( $crawler->{op} ne 'reindex');
   } else {
     warn "Return code:", $page->{code}, ",$uri\n";
   }
   $crawler->mark_url( $uri, ( $page->{success} ) ? $page->{code} : -$page->{code} );
 } else {
   $npage++;
   $crawler->mark_url( $uri, 304 ); # not modified
 }
 print "Processed tid:",$crawler->{tid},",$npage/",$crawler->{maxpages}," pages, links:", scalar keys %{$crawler->{links}},"\r";
 $uri = $crawler->next_url();
}

print "\n";

print "Indexed ",$totalsize," bytes\n";

if ( $opts{dump} ) {
 open(LINKS,">$opts{dump}") || die "Can't open ",$opts{dump}, "for writing !\n";
# $crawler->dump_links(\*LINKS);
 $crawler->dump_links_all(\*LINKS);
 close(LINKS);
}

$crawler->finish();


#########################################################

sub usage {
  my $str = shift;
  print "$str\n" if $str;
  print "Usage: $0 --debug=0|1|2  --db=dbname[:prefix] --dump=links --load=links --maxpages=#pages --op={noindex,reindex,add,test} --seed=seedURL parentURL\n";
  die("\n");
}

sub check_op {
  my ($op, $value) = @_;
  my @vaild_ops = qw(noindex add reindex);
  if ( !grep( /^$value$/, @vaild_ops) ) {
     usage("Invalid op=$value");
  }
  $opts{op} = $value;
#  return $value;
}

=head1 SYNOPSIS

crawler [ --debug={0,1,2} ] [--help] --db=dbname[:prefix] 
        [ --dump=dumplinks --load=links ]
        [ --maxpages=#pages --op={add, noindex, reindex} ]
        [ --seed=seedURL ] parentURL

=item --db=dbname[:prefix]  

specifies database name and prefix of your collection,
should be the same you specified when runing init-crawler.pl
(B<obligatory>)

=item parentURL  

url of parent page. All links outside of parent page will be
rejected. (B<obligatory>)

=item --op={add, noindex, reindex} 

Default value is 'nothing' which assumes indexing is starting from the
very beginning.

=over

=item add

indexed pages will be added to the colection.

=item reindex

re-index pages that already indexed, no new pages will be crawled or indexed.

=item noindex

no real operations with database. Could be useful for link gathering.

=back

=item  --seed=seedURL   

url of seed page, used for starting ( parentURL is used by default )

=item --maxpages=#pages 

maximum number of *indexed* pages ( pages forbidden by users or robots rules 
and failed to fetch don't counted ).

=item --dump=dumplinks 

dump gathered links to the file. Indexed pages will not be dumped. 

==item --load=links 

load gathered links (see --dump). Could be used to indexed them with --op=add 

=cut

=head1 Author

 Oleg Bartunov (oleg@sai.msu.su)

=cut 

=head1 SEE ALSO

 MyCrawler
 Search::OpenFTS::Crawler

=cut
