#!/usr/bin/perl

use strict;

# provide path to your custom dictionaries
use Search::OpenFTS;
use DBI();
use Time::HiRes qw( gettimeofday tv_interval );

my $SEARCH = $ENV{SCRIPT_NAME};
# defaults
my $db = 'www:a'; # database:prefix 
my $pp = 10;      # results per page
# annotation
my $BODYREAD = 8192;  # read $BODYREAD bytes of document from database
my $MAXREAD  = 2048;  # $MAXREAD bytes used to prepare annotation
my $MAXLEN   =  150;  # of length $MAXLEN

my %in = ReadParse();

my ( $checked, $q, $v, $cp, $errstr );

$q = $in{q} || ""; # query
$v = $in{v} ||  0; # verbose
$cp = $in{p}||  1; # current page
$checked = ( $v ) ? 'checked' : '';

         $q =~ s{&}{&amp;}gso;
         $q =~ s{<}{&lt;}gso;
         $q =~ s{>}{&gt;}gso;
         $q =~ s{"}{&quot;}gso;

print "Content-Type: text/html\n\n";

print<<HEADER;
<html>
<head><title>OpenFTS:$q</title>
<style type="text/css">
<!-- 
body {
  margin: 0;
  margin-left: 1em;
  font-family: sans-serif;
  color: black;
  background: white;
}

A:hover {color: red }
DIV.fragments {background-color: #FFFFFF;}
INPUT {
          border:thin solid black;
          margin-bottom: 0px; margin-top: 0px;
};
//-->
</style>
</head>
<body>
<H2 align="center">Search PostgreSQL 7.4.1 documentation</H2>
HEADER

print<<EOFORM;
<form action="$SEARCH" name=f>
<table bgcolor="#6699cc" border=0 cellpadding=0 cellspacing=5>
<tr valign="middle">
<td><img src="img/openfts-logo.gif" height=50 width=50 hspace=0 vspace=0 alt="OpenFTS Home" border=0></td>
<td align="right" nowrap><small>
&nbsp;<input type=text size=40 name="q" value="$q">&nbsp;<input type=submit value="Search">
&nbsp;&nbsp;<input type="button" value="Clear" OnClick="document.f.q.value=\'\';document.f.q.focus()">
<br><a href="$SEARCH">New query</a>&nbsp;&nbsp;<input type="checkbox" name="v" $checked>debug information
</small>
</td>
</tr></table>
</form>

EOFORM

#dump_env();



my ( $dbname, $PREFIX ) = '';
( $dbname, $PREFIX ) = split(':',$db);

my ( $dbh, $fts );
$dbh = DBI->connect('DBI:Pg:dbname='.$dbname);
if ( ! $dbh ) {
  $errstr =  "Couldn't connect to database \'$dbname\' !";
  goto END;
}
$fts = Search::OpenFTS->new( $dbh, prefix=>$PREFIX) || die;
if ( ! $fts ) {
  $errstr =  "Couldn't initialized fts object !";
  goto END;
}

if ( !$q ) {
  print "<script>document.f.q.focus()</script>\n";
  goto END;
}


my (@stopwords,@ignored) = ();
# no check for stopwords if $query passes to get_sql as \@ARGV !!!
my ($out, $condition, $order) = $fts->get_sql( $q, rejected => \@stopwords, ignored=> \@ignored );

if ( $v ) {
  print "Stopwords:",join(',', @stopwords),"\n<BR>" if scalar @stopwords;
# don't show space symbols
#  @ignored = grep { $_->{type} ne 12 } @ignored;
  if ( scalar @ignored ) {
   print "Ignored by parser:<pre>", map {  "(\"$$_{word}\",$$_{type})" } @ignored;
   print "</pre>\n";
  }
}

if ( !$condition ) {
  $errstr = "No documents will be found ! Some of the input words doesn't recognized by dictionaries or *processed* query is empty.";
  goto END;
}

my $limit = $pp;
my $offset = ( $cp-1 ) * $limit;

     my $sql="
select
        $fts->{TXTID},
        $fts->{TABLE}.path,
        substr($fts->{TABLE}.body, 0, $BODYREAD) as body,
        $fts->{TABLE}.title,
        $fts->{TABLE}.dlm,
        $out
from
        $fts->{TABLE}
where
        $condition
order by $order";

$sql .= " limit $limit offset $offset;";

#print "<pre>$sql</pre>\n" if ( $v );
print "<pre>$condition</pre>\n" if ( $v );

my $t0 = [gettimeofday];
my $count = $dbh->selectrow_array("select count(*) from $fts->{TABLE} where $condition;");
my $elapsed = sprintf( "%.3f", tv_interval ( $t0, [gettimeofday]) );

if ( ! $count ) {
  $errstr = "No documents found containing all of the input words !";
  $errstr .= "<BR>".$dbh->errstr if $dbh->errstr;
  goto END;
}

my $sth=$dbh->prepare($sql) || die;
if ( $sth->execute() ) {
# pagination bar
  my ($displayed, $begin, $prev,$next, $end, $np, $prevp, $title, $date, $headline, $tid );
  my $npages = int($count/$pp) + 1;
  $begin = ($npages == 1) ? "" : ( $cp > 1 ) ? "<a href=\"$SEARCH?q=$q&p=1&v=$v\">[1]</a>" : "[1]";
  $np = $cp + 1;
  my $current = " -- <b>$cp</b> -- ";
  $end = ($npages > 1) ? "<a href=\"$SEARCH?q=$q&p=$npages&v=$v\">[$npages]</a>" : "";
  $displayed = $cp*$pp;
  $displayed = $count if ( $displayed > $count );
  $next = "<a href=\"$SEARCH?q=$q&p=$np&v=$v\">&gt;&gt</a>" if $count > $displayed;
  $prevp = $cp - 1;
  $prev = "<a href=\"$SEARCH?q=$q&p=$prevp&v=$v\">&lt;&lt;</a>" if $cp >1;
  my $i = $offset+1;
  my $bar="<small>Documents <b>$i-$displayed</b> from <b>$count</b> ($elapsed sec.). Pages: $begin $prev $current $next $end </small>";

  print "$bar\n<HR><small><dl>\n";

  while ( defined ( my $r=$sth->fetchrow_hashref ) ) {
     $tid   = $r->{tid};
     $title = $r->{title};
     $title ||= $r->{path};
     $date = localtime($r->{dlm});
     print "<dt>$i.&nbsp;<a href=\"$r->{path}\">$title</a> (",sprintf("%.3f",$r->{pos}),")\n"; 
     $headline=$fts->get_headline(query=>$q, src=>\$r->{body}, 
                                  maxread=>$MAXREAD, maxlen=>$MAXLEN,
                                  replace_ignore_headline=>' ',
                                  otag=>'<b>',ctag=>'</b>' );
     $headline =~ s/(\s+)/ /g;
     print "<dd><div class=\"fragments\">$headline</div>\n<font color=\"#6f6f6f\">$date</font>";
     print "&nbsp;(<a href=\"cache.cgi?tid=$tid\" TARGET=\"_blank\">cached</a>)\n";
     $i++;
  }
  print "</dl></small>$bar\n";
} else {
  $errstr = "Sorry, there is a problem with your query!<BR>",
            "Query:<pre> $sql</pre>\n<BR>",$dbh->errstr;
}
$sth->finish if $sth;

END:

print "$errstr<BR>\n" if $errstr;

print<<FOOTER;
<div align="center">
<hr>
<small>
Powered by <a href="http://openfts.sourceforge.net">OpenFTS</a>&nbsp;v. $fts->{VERSION}</div>
</small>
</body>
</html>
FOOTER

$dbh->disconnect if $dbh;

#########################################################################
sub ReadParse {
  my %in = ();
  my $in = '';
  if ( $ENV{'REQUEST_METHOD'} eq 'GET') {
     $in = $ENV{'QUERY_STRING'};
  }elsif ($ENV{'REQUEST_METHOD'} eq 'POST') {
     for (my $i = 0; $i < $ENV{'CONTENT_LENGTH'}; $i++) {
	$in .= getc;
     }
  } elsif ($ENV{'REQUEST_METHOD'} eq "") { # for off-line testing...
                $in = $ARGV[0];
  }


  # use this for multiple fields (see below)
  my $null = "\000";

  my ( $loc, $key, $val);
  foreach my $var ( split(/&/,$in) ) {
	# Convert plus's to spaces
	$var =~ s/\+/ /g;
	# Convert %XX from hex numbers to alphanumeric
	$var =~ s/%(..)/pack("c",hex($1))/ge;
	# Split into key and value.;
	$loc = index($var,"=");
	$key = substr($var,0,$loc);
	$val = substr($var,$loc+1);
	$in{$key} .= $null if (defined($in{$key})); # \0 is the multiple separator
	$in{$key} .= $val;
  }
  return %in;
}

sub dump_env {
   map { print "$_ = $ENV{$_}<BR>\n"; } sort keys %ENV;
}
