#!/usr/bin/perl
#Example of init new data base
use strict;

# provide path to your custom dictionaries
use lib qw (/u/megera/app/fts/test-suite);
use Search::OpenFTS::Index;
use DBI();

my $TABLE = 'pages';

if ( $#ARGV < 0 ) {
	print "Usage:\n$0 DATABASE[:PREFIX]\n contrib/tsearch should be installed!\n";
	print "Drop OpenFTS instance: $0 DATABASE[:PREFIX] drop\n"; 
	exit;
}

my ( $dbname, $PREFIX ) = '';
( $dbname, $PREFIX ) = split(':',$ARGV[0]);


my $dbi =DBI->connect('DBI:Pg:dbname='.$dbname);
$dbi || die;

if ( $ARGV[1] eq 'drop' ) {
	my $idx=Search::OpenFTS::Index->new($dbi, $PREFIX) || die;
	$idx->drop;

	$dbi->do("drop table $idx->{TABLE};") || warn "Can't drop table pages";
	$dbi->disconnect;

	exit;
}


my $TPREFIX = $PREFIX."_" if ( $PREFIX );

my $DICT_UNKNOWN_LEXEM_TABLE = $TPREFIX.'fts_unknown_lexem';

$dbi->do("create table ${TPREFIX}$TABLE (
        tid int not null primary key,
        path varchar not null unique,
        body varchar,
        title varchar,
        di int4,
        dlm int4,
        de int4,
        md5 char(22),
        fts_index tsvector
);") || die;

{
my $idx=Search::OpenFTS::Index->init( 
	dbi=>$dbi, 
        prefix=>$PREFIX,
	txttid=>${TPREFIX}.$TABLE.'.tid',
	tsvector_field=>'fts_index',
        numbergroup=>10,
	ignore_id_index=>[ qw(7 13 14 12    8 9 15 18 19) ],
	ignore_headline=>[ qw(13 15 16 17 5) ],
	map=>'{ 
               \'20\'=>[1], 21=>[2], 22=>[2],        # numbers
                1=>[0], 11=>[0], 16=>[0],      	     # latin
                3=>[3], 4=>[3],  5=>[3], 3=>[3], 8=>[3], 18=>[3], 19=>[3], # unknown
              }',
        dict=>[
# example how to use snowball stemmer
#                { mod=>'Search::OpenFTS::Morph::ISpell', param=>'{aff_file=>"/usr/local/lib/english.aff",dict_file=>"/usr/local/lib/english.dict",stop_file=>"/u/megera/app/fts/test-suite/Dict/english.stop" }' },
                { mod=>'Search::OpenFTS::Dict::Snowball', param=>'{lang=>"english", stop_file=>"/u/megera/app/fts/pgsql/english.stop"}' },
                { mod=>'Dict::DecimalDict', param=>'{MAXLENFRAC=>1,REJECTLONG=>0}' },
                { mod=>'Dict::IntegerDict', param=>'{MAXLEN    =>6,REJECTLONG=>0}' },
                { mod=>'Search::OpenFTS::Dict::UnknownDict', param=>"{table => \'$DICT_UNKNOWN_LEXEM_TABLE\'}" },
	] 
);
die "QQQ: $@" if ! $idx;

	# here we could index files using method $idx->index;
	# it should be quite fast, because indices doesn't created yet

$idx->create_index; #this function called automatically when destroy object $idx 

# grant permissions on search tables and indices to PUBLIC
my $grant = $idx->fix_permissions();
print "Permissions (select) granted on search tables and indices to PUBLIC\n" if ( $grant == 1);

}
$dbi->disconnect;
